'use strict';

var _getIterator2 = require('babel-runtime/core-js/get-iterator');

var _getIterator3 = _interopRequireDefault(_getIterator2);

var _asyncIterator2 = require('babel-runtime/helpers/asyncIterator');

var _asyncIterator3 = _interopRequireDefault(_asyncIterator2);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

let fetchIssues = (() => {
  var _ref = (0, _asyncToGenerator3.default)(function* (config, github) {
    const state = determineFetchState(config);
    return config.pullRequests ? github.fetchPullRequestsSince(config.startAt, state) : github.fetchIssuesSince(config.startAt, state);
  });

  return function fetchIssues(_x, _x2) {
    return _ref.apply(this, arguments);
  };
})();

let git3po = (() => {
  var _ref2 = (0, _asyncToGenerator3.default)(function* (options) {
    const log = options.log,
          configPath = options.configPath,
          dryRun = options.dryRun;

    const config = Config.from(configPath);
    const filters = config.filters;
    const actions = config.actions;

    const github = new GitHubApi(config.githubApiBaseUrl, config.token, config.repo, dryRun);

    const issuePager = yield fetchIssues(config, github);
    var _iteratorNormalCompletion = true;
    var _didIteratorError = false;
    var _iteratorError = undefined;

    try {
      for (var _iterator = (0, _asyncIterator3.default)(issuePager), _step, _value; _step = yield _iterator.next(), _iteratorNormalCompletion = _step.done, _value = yield _step.value, !_iteratorNormalCompletion; _iteratorNormalCompletion = true) {
        const issues = _value;
        var _iteratorNormalCompletion2 = true;
        var _didIteratorError2 = false;
        var _iteratorError2 = undefined;

        try {
          for (var _iterator2 = (0, _getIterator3.default)(issues), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
            const issue = _step2.value;

            let passes = true;
            var _iteratorNormalCompletion3 = true;
            var _didIteratorError3 = false;
            var _iteratorError3 = undefined;

            try {
              for (var _iterator3 = (0, _getIterator3.default)(filters), _step3; !(_iteratorNormalCompletion3 = (_step3 = _iterator3.next()).done); _iteratorNormalCompletion3 = true) {
                const filter = _step3.value;

                // eslint-disable-next-line no-await-in-loop
                passes = yield filter.apply(issue, github);
                if (!passes) {
                  const issueMsg = `Issue #${issue.number}`;
                  log(`${'⤺'.yellow}  ${issueMsg} did not match ${filter.toLog()} filter, skipping`);
                  break;
                }
              }
            } catch (err) {
              _didIteratorError3 = true;
              _iteratorError3 = err;
            } finally {
              try {
                if (!_iteratorNormalCompletion3 && _iterator3.return) {
                  _iterator3.return();
                }
              } finally {
                if (_didIteratorError3) {
                  throw _iteratorError3;
                }
              }
            }

            if (!passes) {
              continue;
            }

            const startMsgDesc = `(${issue.title}) at ${issue.created_at}`;
            log(`💎  Found #${issue.number} ${startMsgDesc}, processing...`);
            log(`🔗  https://github.com/${config.repo}/issues/${issue.number}`);

            var _iteratorNormalCompletion4 = true;
            var _didIteratorError4 = false;
            var _iteratorError4 = undefined;

            try {
              for (var _iterator4 = (0, _getIterator3.default)(actions), _step4; !(_iteratorNormalCompletion4 = (_step4 = _iterator4.next()).done); _iteratorNormalCompletion4 = true) {
                const action = _step4.value;

                const actionLog = _.padEnd(`     Applying ${action.toLog()}...`, 40);
                log(actionLog, '⏳');

                // eslint-disable-next-line no-await-in-loop
                yield action.apply(issue, github);
              }
            } catch (err) {
              _didIteratorError4 = true;
              _iteratorError4 = err;
            } finally {
              try {
                if (!_iteratorNormalCompletion4 && _iterator4.return) {
                  _iterator4.return();
                }
              } finally {
                if (_didIteratorError4) {
                  throw _iteratorError4;
                }
              }
            }

            log(`✅  Done with #${issue.number}`);
          }
        } catch (err) {
          _didIteratorError2 = true;
          _iteratorError2 = err;
        } finally {
          try {
            if (!_iteratorNormalCompletion2 && _iterator2.return) {
              _iterator2.return();
            }
          } finally {
            if (_didIteratorError2) {
              throw _iteratorError2;
            }
          }
        }
      }
    } catch (err) {
      _didIteratorError = true;
      _iteratorError = err;
    } finally {
      try {
        if (!_iteratorNormalCompletion && _iterator.return) {
          yield _iterator.return();
        }
      } finally {
        if (_didIteratorError) {
          throw _iteratorError;
        }
      }
    }
  });

  return function git3po(_x3) {
    return _ref2.apply(this, arguments);
  };
})();

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

require('colors');

const _ = require('lodash');
const Filter = require('./filter');
const Config = require('./config');
const GitHubApi = require('./github');

function determineFetchState(config) {
  let state = 'all';

  const stateFilter = config.filters.find(filter => filter.type === Filter.TYPES.STATE);
  const issueFilter = config.filters.find(filter => filter.type === Filter.TYPES.ISSUE);
  const stateCondition = _.get(issueFilter, ['_filter', 'criteria', 'state']);
  if (stateFilter) {
    state = stateFilter.apply({ state: 'open' }) ? 'open' : 'closed';
  } else if (typeof stateCondition === 'string') {
    state = stateCondition;
  }

  return state;
}

module.exports = { git3po, fetchIssues };