'use strict';

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _keys = require('babel-runtime/core-js/object/keys');

var _keys2 = _interopRequireDefault(_keys);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const _ = require('lodash');
const Filter = require('../filter');

const SUPPORTED_FILTERS = ['text', 'author'];
const FILTER_GETTERS = {
  text: 'body',
  author: 'user.login'
};

class CommentsFilter extends Filter {
  get type() {
    return Filter.TYPES.COMMENTS;
  }

  static createCommentPredicate(filterDefinition) {
    const commentPredicates = _.intersection((0, _keys2.default)(filterDefinition), SUPPORTED_FILTERS).map(name => {
      const valuePredicate = Filter.createPredicate(filterDefinition[name]);
      const valueGetter = comment => _.get(comment, FILTER_GETTERS[name]);
      return comment => valuePredicate(valueGetter(comment));
    });

    return comment => commentPredicates.every(predicate => predicate(comment));
  }

  _apply(issue, github) {
    var _this = this;

    return (0, _asyncToGenerator3.default)(function* () {
      const comments = yield github.fetchComments(issue);

      const commentPredicate = CommentsFilter.createCommentPredicate(_this._filter);
      const filteredComments = comments.filter(commentPredicate);

      const lengthFilter = typeof _this._filter.length === 'undefined' ? { $gt: 0 } : _this._filter.length;
      const lengthPredicate = Filter.createPredicate(lengthFilter);
      return lengthPredicate(filteredComments.length);
    })();
  }
}

module.exports = CommentsFilter;