'use strict';

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const _ = require('lodash');
const GitHub = require('../github');
const Filter = require('../filter');

class LatestActivityFilter extends Filter {
  get type() {
    return Filter.TYPES.LATEST_ACTIVITY;
  }

  _apply(issue, github) {
    var _this = this;

    return (0, _asyncToGenerator3.default)(function* () {
      let activities = yield _promise2.default.all([github.fetchComments(issue), github.fetchCommits(issue)]);

      activities = _.flatten(activities).map(GitHub.transformCommentOrCommit);
      if (_this._filter.filter) {
        const filter = Filter.preprocess(_this._filter.filter, issue);
        activities = activities.filter(Filter.createPredicate(filter));
      }

      const mostRecentActivity = _.maxBy(activities, function (activity) {
        return activity.createdAt.getTime();
      });
      const criteria = Filter.preprocess(_this._filter.criteria, issue);
      const predicate = Filter.createPredicate(criteria);
      return predicate(mostRecentActivity);
    })();
  }
}

module.exports = LatestActivityFilter;